package cn.com.qmth.examcloud.core.oe.task.service.pipeline;

import cn.com.qmth.examcloud.commons.helpers.KeyValuePair;
import cn.com.qmth.examcloud.commons.helpers.ObjectHolder;
import cn.com.qmth.examcloud.commons.helpers.pipeline.NodeExecuter;
import cn.com.qmth.examcloud.commons.helpers.pipeline.TaskContext;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamScoreNoticeQueueCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.SyncExamDataCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamCaptureBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamContinuedRecordBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamFaceLivenessVerifyBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamProcessRecordBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamQuestionBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordDataBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordPaperStructBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordQuestionsBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamSyncCaptureBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.FaceBiopsyBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.FaceBiopsyItemBean;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.FaceBiopsyItemStepBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.AddExamScoreNoticeQueueReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.SyncExamDataReq;
import cn.com.qmth.examcloud.core.oe.student.api.ExamRecordDataCloudService;
import cn.com.qmth.examcloud.core.oe.student.api.bean.StuExamQuestionBean;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamFaceLivenessVerifiesReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamRecordPaperStructReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamRecordQuestionsReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetFaceBiopsyReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.UpdatePartialExamRecordReq;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetExamFaceLivenessVerifiesResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetExamRecordPaperStructResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetExamRecordQuestionsResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetFaceBiopsyResp;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamContinuedRecordRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamProcessRecordRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamContinuedRecordEntity;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamProcessRecordEntity;
import cn.com.qmth.examcloud.core.oe.task.dao.ExamCaptureRepo;
import cn.com.qmth.examcloud.core.oe.task.dao.ExamSyncCaptureRepo;
import cn.com.qmth.examcloud.core.oe.task.dao.entity.ExamCaptureEntity;
import cn.com.qmth.examcloud.core.oe.task.dao.entity.ExamSyncCaptureEntity;
import cn.com.qmth.examcloud.core.oe.task.service.ExamBossService;
import cn.com.qmth.examcloud.core.oe.task.service.ExamRecordDataService;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.enums.SyncStatus;
import cn.com.qmth.examcloud.support.examing.ExamBoss;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.support.helper.FaceBiopsyHelper;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description 同步考试数据执行器
 * @Author lideyin
 * @Date 2019/12/19 16:22
 * @Version 1.0
 */
@Component
public class SyncExamDataExecutor implements NodeExecuter<Long, ExamRecordData, Long, ExamRecordData> {

    @Autowired
    private SyncExamDataCloudService syncExamDataCloudService;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Autowired
    private ExamCaptureRepo examCaptureRepo;

    @Autowired
    private ExamSyncCaptureRepo examSyncCaptureRepo;

    @Autowired
    private ExamRecordDataCloudService examRecordDataCloudService;

    @Autowired
    private ExamScoreNoticeQueueCloudService examScoreNoticeQueueCloudService;

    @Autowired
    private ExamBossService examBossService;

    @Autowired
    private ExamContinuedRecordRepo examContinuedRecordRepo;

    @Autowired
    private ExamProcessRecordRepo examProcessRecordRepo;

    private static final Logger LOG = LoggerFactory.getLogger(SyncExamDataExecutor.class);

    /**
     * 执行
     *
     * @param key
     * @param examRecordData
     * @param outList
     * @param removable
     * @param context
     * @throws Exception
     * @author WANGWEI
     */
    @Override
    public void execute(Long key, ExamRecordData examRecordData,
                        List<KeyValuePair<Long, ExamRecordData>> outList,
                        ObjectHolder<Boolean> removable, TaskContext context) throws Exception {

        Long studentId = examRecordData.getStudentId();
        String sequenceLockKey = Constants.EXAM_CONTROL_LOCK_PREFIX + studentId;

        long st = System.currentTimeMillis();

        try {
            debugLog("enter executor...", examRecordData.getId());

            //添加考试控制全局锁
            SequenceLockHelper.getLockSimple(sequenceLockKey);

            debugLog("get locker success...", examRecordData.getId());

            //如果已同步，直接交给下一节点
            if (SyncStatus.SYNCED == examRecordData.getSyncStatus()) {

                outList.add(new KeyValuePair<>(key, examRecordData));

                this.debugLog("already synced,will go to next node...", examRecordData.getId());

                return;
            }

            //处理上一节点中，指定时间内仍未处理完成的数据
            Long examRecordDataId = examRecordData.getId();
            if (ExamRecordStatus.EXAM_HAND_IN == examRecordData.getExamRecordStatus() ||
                    ExamRecordStatus.EXAM_AUTO_HAND_IN == examRecordData.getExamRecordStatus()) {

                this.debugLog("prepare to execute last node's hand in data...", examRecordDataId);
                long startTime = System.currentTimeMillis();
                examRecordData = examRecordDataService.processAfterHandInExam(examRecordDataId);

                this.debugLog("T001.execute last node's hand in data success...耗时：" +
                        (System.currentTimeMillis() - startTime) + " ms", examRecordDataId);
            }

            //如果考试记录状态为完结状态，则开始同步数据到正式库
            if (ExamRecordStatus.EXAM_END == examRecordData.getExamRecordStatus() ||
                    ExamRecordStatus.EXAM_OVERDUE == examRecordData.getExamRecordStatus()) {

                this.debugLog("prepare to sync data...", examRecordDataId);

                //同步数据
                SyncExamDataReq syncReq = new SyncExamDataReq();

                syncReq.setExamRecordPaperStruct(getExamRecordPaperStruct(examRecordDataId));

                syncReq.setExamRecordQuestions(getExamRecordQuestions(examRecordDataId));

                syncReq.setExamContinuedRecords(getExamContinuedRecords(examRecordDataId));

                syncReq.setExamProcessRecords(getExamProcessRecords(examRecordDataId));

                //开启人脸检测相关数据赋值
                Long rootOrgId = examRecordData.getRootOrgId();
                Long examId = examRecordData.getExamId();
                if (FaceBiopsyHelper.isFaceEnable(rootOrgId, examId, studentId)) {
                    List<ExamCaptureBean> captures = getExamCaptures(examRecordDataId);
                    syncReq.setExamCaptures(captures);

                    ExamSyncCaptureBean syncCapture = getExamSyncCapture(examRecordDataId);
                    syncReq.setExamSyncCapture(syncCapture);

                    //虚拟摄像头进入待审核，且有虚拟摄像头的或者同步没有照片的，更新缓存
                    boolean hasVirtualCamera = captures.stream().anyMatch(e->e.getHasVirtualCamera()!=null&&e.getHasVirtualCamera());
                    if (examRecordDataService.isVirtualToWaiting(examId) && hasVirtualCamera || syncCapture == null) {
                        examRecordData.setIsWarn(true);
                        examRecordDataService.saveExamRecordDataCache(examRecordDataId, examRecordData);
                    }

                    if (FaceBiopsyHelper.isFaceVerify(rootOrgId, examId, studentId)) {
                        syncReq.setExamFaceLivenessVerifies(getExamFaceLivenessVerifies(examRecordDataId));

                        syncReq.setFaceBiopsy(getFaceBiopsy(examRecordDataId));
                    }
                }

                syncReq.setExamRecordData(copyExamRecordDataFrom(examRecordData));

                long startTime = System.currentTimeMillis();

                //同步数据
                syncExamDataCloudService.syncExamData(syncReq);

                this.debugLog("T002.同步数据rpc耗时:" + (System.currentTimeMillis() - startTime) + " ms", examRecordDataId);

                this.debugLog("sync data success,and to be change status...", examRecordDataId);

                //考试完结次数加1
                ExamBoss examBoss = examBossService.getExamBoss(examRecordData.getExamStudentId());
                if (null != examBoss) {
                    examBoss.setEndCount(examBoss.getEndCount() + 1);

                    startTime = System.currentTimeMillis();

                    examBossService.saveExamBoss(examRecordData.getExamStudentId(), examBoss);

                    this.debugLog("T003.考试完结次数加1,并saveExamBoss耗时..." +
                            (System.currentTimeMillis() - startTime) + " ms", examRecordDataId);
                }

                //设置并保存考试记录的同步状态
                startTime = System.currentTimeMillis();

                examRecordData.setSyncStatus(SyncStatus.SYNCED);
                setAndSaveExamRecordDataSyncStatus(examRecordDataId);
                this.debugLog("T004.设置并保存考试记录的同步状态,耗时..." +
                        (System.currentTimeMillis() - startTime) + " ms", examRecordDataId);

                //保存成绩通知队列
                String isPushScore = ExamCacheTransferHelper.getCachedExamProperty(examRecordData.getExamId(),
                        examRecordData.getStudentId(), ExamProperties.PUSH_SCORE.name()).getValue();
                if (Constants.isTrue.equals(isPushScore)) {
                    AddExamScoreNoticeQueueReq req = new AddExamScoreNoticeQueueReq();
                    req.setRootOrgId(rootOrgId);
                    examScoreNoticeQueueCloudService.addExamScoreNoticeQueue(req);
                }

                outList.add(new KeyValuePair<>(key, examRecordData));

                this.debugLog("sync data and change status success...", examRecordDataId);

                return;
            }

            //其它状态的数据，直接交给下一个节点处理
            outList.add(new KeyValuePair<>(key, examRecordData));

            this.debugLog("current status is '" + examRecordData.getExamRecordStatus().name() +
                    "'.do nothing and go to the next node...", examRecordData.getId());
        } finally {
            SequenceLockHelper.releaseLockSimple(sequenceLockKey);
            this.debugLog("T999 同步数据合计耗时:" + (System.currentTimeMillis() - st) + " ms", examRecordData.getId());
        }
    }

    private List<ExamContinuedRecordBean> getExamContinuedRecords(Long examRecordDataId) {
        List<ExamContinuedRecordEntity> entityList = examContinuedRecordRepo.findByExamRecordDataId(examRecordDataId);
        if (null == entityList || entityList.isEmpty()) {
            return new ArrayList<>();
        }

        List<ExamContinuedRecordBean> resultList = new ArrayList<>();
        for (ExamContinuedRecordEntity entity : entityList) {
            ExamContinuedRecordBean bean = new ExamContinuedRecordBean();
            bean.setId(entity.getId());
            bean.setExamRecordDataId(entity.getExamRecordDataId());
            bean.setContinuedTime(entity.getContinuedTime());
            bean.setStartTime(entity.getStartTime());
            bean.setUsedExamTime(entity.getUsedExamTime());

            resultList.add(bean);
        }

        return resultList;
    }

    private List<ExamProcessRecordBean> getExamProcessRecords(Long examRecordDataId) {
        List<ExamProcessRecordEntity> entityList = examProcessRecordRepo.findByExamRecordDataId(examRecordDataId);
        if (null == entityList || entityList.isEmpty()) {
            return new ArrayList<>();
        }

        List<ExamProcessRecordBean> resultList = new ArrayList<>();
        for (ExamProcessRecordEntity entity : entityList) {
            ExamProcessRecordBean bean = new ExamProcessRecordBean();
            bean.setId(entity.getId());
            bean.setExamRecordDataId(entity.getExamRecordDataId());
            bean.setProcessName(entity.getProcessName());
            bean.setRecordTime(entity.getRecordTime());
            bean.setSourceIp(entity.getSourceIp());

            resultList.add(bean);
        }

        return resultList;
    }

    private FaceBiopsyBean getFaceBiopsy(Long examRecordDataId) {
        GetFaceBiopsyReq req = new GetFaceBiopsyReq();
        req.setExamRecordDataId(examRecordDataId);

        GetFaceBiopsyResp resp = examRecordDataCloudService.getFaceBiopsy(req);

        if (null == resp.getFaceBiopsyBean()) {
            return null;
        }

        cn.com.qmth.examcloud.core.oe.student.api.bean.FaceBiopsyBean faceBiopsyBean = resp.getFaceBiopsyBean();

        FaceBiopsyBean bean = new FaceBiopsyBean();
        bean.setErrorMsg(faceBiopsyBean.getErrorMsg());
        bean.setExamRecordDataId(faceBiopsyBean.getExamRecordDataId());
        bean.setResult(faceBiopsyBean.getResult());
        bean.setRootOrgId(faceBiopsyBean.getRootOrgId());
        bean.setVerifiedTimes(faceBiopsyBean.getVerifiedTimes());

        List<FaceBiopsyItemBean> faceBiopsyItemList = new ArrayList<>();

        for (cn.com.qmth.examcloud.core.oe.student.api.bean.FaceBiopsyItemBean faceBiopsyItemBean : faceBiopsyBean.getFaceBiopsyItems()) {
            FaceBiopsyItemBean itemBean = new FaceBiopsyItemBean();

            itemBean.setCompleted(faceBiopsyItemBean.getCompleted());
            itemBean.setErrorMsg(faceBiopsyItemBean.getErrorMsg());
            itemBean.setExamRecordDataId(faceBiopsyItemBean.getExamRecordDataId());
            itemBean.setFaceBiopsyId(faceBiopsyItemBean.getFaceBiopsyId());
            itemBean.setFaceBiopsyType(faceBiopsyItemBean.getFaceBiopsyType());
            itemBean.setInFreezeTime(faceBiopsyItemBean.getInFreezeTime());
            itemBean.setResult(faceBiopsyItemBean.getResult());

            List<FaceBiopsyItemStepBean> itemStepList = new ArrayList<>();
            for (cn.com.qmth.examcloud.core.oe.student.api.bean.FaceBiopsyItemStepBean faceBiopsyItemStepBean : faceBiopsyItemBean.getFaceBiopsyItemSteps()) {
                FaceBiopsyItemStepBean itemStepBean = new FaceBiopsyItemStepBean();
                itemStepBean.setAction(faceBiopsyItemStepBean.getAction());
                itemStepBean.setActionStay(faceBiopsyItemStepBean.getActionStay());
                itemStepBean.setErrorMsg(faceBiopsyItemStepBean.getErrorMsg());
                itemStepBean.setExamRecordDataId(faceBiopsyItemStepBean.getExamRecordDataId());
                itemStepBean.setFaceBiopsyItemId(faceBiopsyItemStepBean.getFaceBiopsyItemId());
                itemStepBean.setResourceRelativePath(faceBiopsyItemStepBean.getResourceRelativePath());
                itemStepBean.setResourceType(faceBiopsyItemStepBean.getResourceType());
                itemStepBean.setResult(faceBiopsyItemStepBean.getResult());

                itemStepList.add(itemStepBean);
            }

            itemBean.setFaceBiopsyItemSteps(itemStepList);

        }

        bean.setFaceBiopsyItems(faceBiopsyItemList);

        return bean;

    }

    private ExamSyncCaptureBean getExamSyncCapture(Long examRecordDataId) {
        ExamSyncCaptureEntity entity = examSyncCaptureRepo.findByExamRecordDataId(examRecordDataId);
        if (null == entity) {
            return null;
        }

        ExamSyncCaptureBean bean = new ExamSyncCaptureBean();
        bean.setId(entity.getId());
        bean.setExamRecordDataId(entity.getExamRecordDataId());
        bean.setFileUrl(entity.getFileUrl());
        bean.setFileName(entity.getFileName());
        bean.setPass(entity.getIsPass());
        bean.setFaceCompareResult(entity.getFaceCompareResult());
        bean.setStranger(entity.getIsStranger());
        bean.setLandmark(entity.getLandmark());
        bean.setFacelivenessResult(entity.getFacelivenessResult());
        bean.setUsedTime(entity.getUsedTime());
        bean.setProcessTime(entity.getProcessTime());
        bean.setHasVirtualCamera(entity.getHasVirtualCamera());
        bean.setCameraInfos(entity.getCameraInfos());
        bean.setExtMsg(entity.getExtMsg());

        return bean;
    }

    private ExamRecordQuestionsBean getExamRecordQuestions(Long examRecordDataId) {
        GetExamRecordQuestionsReq req = new GetExamRecordQuestionsReq();
        req.setExamRecordDataId(examRecordDataId);
        GetExamRecordQuestionsResp resp = examRecordDataCloudService.getExamRecordQuestions(req);

        ExamRecordQuestionsBean bean = new ExamRecordQuestionsBean();
        bean.setCreationTime(resp.getCreationTime());
        bean.setExamRecordDataId(resp.getExamRecordDataId());

        List<ExamQuestionBean> examQuestionBeanList = new ArrayList<>();
        for (StuExamQuestionBean stuExamQuestionBean : resp.getExamQuestions()) {
            ExamQuestionBean examQuestionBean = new ExamQuestionBean();
            examQuestionBean.setExamRecordDataId(stuExamQuestionBean.getExamRecordDataId());
            examQuestionBean.setMainNumber(stuExamQuestionBean.getMainNumber());
            examQuestionBean.setQuestionId(stuExamQuestionBean.getQuestionId());
            examQuestionBean.setOrder(stuExamQuestionBean.getOrder());
            examQuestionBean.setQuestionScore(stuExamQuestionBean.getQuestionScore());
            examQuestionBean.setQuestionType(stuExamQuestionBean.getQuestionType());
            examQuestionBean.setCorrectAnswer(stuExamQuestionBean.getCorrectAnswer());
            examQuestionBean.setStudentAnswer(stuExamQuestionBean.getStudentAnswer());
            examQuestionBean.setStudentScore(stuExamQuestionBean.getStudentScore());
            examQuestionBean.setAnswer(stuExamQuestionBean.getAnswer());
            examQuestionBean.setSign(stuExamQuestionBean.getSign());
            examQuestionBean.setOptionPermutation(stuExamQuestionBean.getOptionPermutation());
            examQuestionBean.setAudioPlayTimes(stuExamQuestionBean.getAudioPlayTimes());

            if (null != stuExamQuestionBean.getAnswerType()) {
                examQuestionBean.setAnswerType(stuExamQuestionBean.getAnswerType().name());
            }

            examQuestionBeanList.add(examQuestionBean);
        }

        bean.setExamQuestionBeans(examQuestionBeanList);

        return bean;
    }

    private ExamRecordPaperStructBean getExamRecordPaperStruct(Long examRecordDataId) {
        GetExamRecordPaperStructReq req = new GetExamRecordPaperStructReq();
        req.setExamRecordDataId(examRecordDataId);
        GetExamRecordPaperStructResp resp = examRecordDataCloudService.getExamRecordPaperStruct(req);

        ExamRecordPaperStructBean bean = new ExamRecordPaperStructBean();
        bean.setId(resp.getId());
        bean.setDefaultPaper(resp.getDefaultPaper());

        return bean;
    }

    private List<ExamFaceLivenessVerifyBean> getExamFaceLivenessVerifies(Long examRecordDataId) {
        GetExamFaceLivenessVerifiesReq req = new GetExamFaceLivenessVerifiesReq();
        req.setExamRecordDataId(examRecordDataId);

        GetExamFaceLivenessVerifiesResp resp = examRecordDataCloudService.getExamFaceLivenessVerifies(req);
        if (null == resp.getExamFaceLivenessVerifis() || resp.getExamFaceLivenessVerifis().isEmpty()) {
            return new ArrayList<>();
        }

        debugLog("the result joson of method getExamFaceLivenessVerifies is :" + JsonUtil.toJson(resp), examRecordDataId);

        List<ExamFaceLivenessVerifyBean> resultList = new ArrayList<>();
        for (cn.com.qmth.examcloud.core.oe.student.api.bean.ExamFaceLivenessVerifyBean eflvb : resp.getExamFaceLivenessVerifis()) {
            ExamFaceLivenessVerifyBean bean = new ExamFaceLivenessVerifyBean();
            bean.setId(eflvb.getId());
            bean.setExamRecordDataId(eflvb.getExamRecordDataId());
            bean.setStartTime(eflvb.getStartTime());
            bean.setUsedTime(eflvb.getUsedTime());
            bean.setResultJson(eflvb.getResultJson());
            bean.setVerifyResult(eflvb.getVerifyResult());
            bean.setBizId(eflvb.getBizId());
            bean.setIsError(eflvb.getIsError());
            bean.setErrorMsg(eflvb.getErrorMsg());
            bean.setOperateNum(eflvb.getOperateNum());

            resultList.add(bean);
        }

        return resultList;
    }

    private List<ExamCaptureBean> getExamCaptures(Long examRecordDataId) {
        List<ExamCaptureEntity> entityList = examCaptureRepo.findByExamRecordDataId(examRecordDataId);
        if (null == entityList || entityList.isEmpty()) {
            return new ArrayList<>();
        }

        List<ExamCaptureBean> resultList = new ArrayList<>();
        for (ExamCaptureEntity entity : entityList) {
            ExamCaptureBean bean = new ExamCaptureBean();

            bean.setId(entity.getId());
            bean.setExamRecordDataId(entity.getExamRecordDataId());
            bean.setFileUrl(entity.getFileUrl());
            bean.setFileName(entity.getFileName());
            bean.setPass(entity.getIsPass());
            bean.setFaceCompareResult(entity.getFaceCompareResult());
            bean.setStranger(entity.getIsStranger());
            bean.setLandmark(entity.getLandmark());
            bean.setFacelivenessResult(entity.getFacelivenessResult());
            bean.setUsedTime(entity.getUsedTime());
            bean.setProcessTime(entity.getProcessTime());
            bean.setHasVirtualCamera(entity.getHasVirtualCamera());
            bean.setCameraInfos(entity.getCameraInfos());
            bean.setExtMsg(entity.getExtMsg());

            resultList.add(bean);
        }

        return resultList;
    }

    private ExamRecordDataBean copyExamRecordDataFrom(ExamRecordData examRecordData) {
        ExamRecordDataBean data = new ExamRecordDataBean();
        data.setId(examRecordData.getId());
        data.setExamId(examRecordData.getExamId());
        data.setExamType(examRecordData.getExamType() == null ? null : examRecordData.getExamType().toString());
        data.setExamStudentId(examRecordData.getExamStudentId());
        data.setStudentId(examRecordData.getStudentId());
        data.setCourseId(examRecordData.getCourseId());
        data.setOrgId(examRecordData.getOrgId());
        data.setRootOrgId(examRecordData.getRootOrgId());
        data.setBasePaperId(examRecordData.getBasePaperId());
        data.setPaperType(examRecordData.getPaperType());
        data.setExamRecordStatus(examRecordData.getExamRecordStatus() == null ? null : examRecordData.getExamRecordStatus().toString());
        data.setStartTime(examRecordData.getStartTime());
        data.setEndTime(examRecordData.getEndTime());
        data.setCleanTime(examRecordData.getCleanTime());
        data.setWarn(examRecordData.getIsWarn() == null ? false : examRecordData.getIsWarn());
        data.setAudit(examRecordData.getIsAudit() == null ? false : examRecordData.getIsAudit());
        data.setIllegality(examRecordData.getIsIllegality() == null ? false : examRecordData.getIsIllegality());
        data.setUsedExamTime(examRecordData.getUsedExamTime());
        data.setContinued(examRecordData.getIsContinued() == null ? false : examRecordData.getIsContinued());
        data.setContinuedCount(examRecordData.getContinuedCount());
        data.setExceed(examRecordData.getIsExceed() == null ? false : examRecordData.getIsExceed());
        data.setFaceSuccessCount(examRecordData.getFaceSuccessCount());
        data.setFaceFailedCount(examRecordData.getFaceFailedCount());
        data.setFaceStrangerCount(examRecordData.getFaceStrangerCount());
        data.setFaceTotalCount(examRecordData.getFaceTotalCount());
        data.setFaceSuccessPercent(examRecordData.getFaceSuccessPercent());
        data.setFaceVerifyResult(examRecordData.getFaceVerifyResult() == null ? null : examRecordData.getFaceVerifyResult().toString());
        data.setBaiduFaceLivenessSuccessPercent(examRecordData.getBaiduFaceLivenessSuccessPercent());
        data.setTotalScore(examRecordData.getTotalScore());
        data.setObjectiveScore(examRecordData.getObjectiveScore());
        data.setObjectiveAccuracy(examRecordData.getObjectiveAccuracy());
        data.setSubjectiveScore(examRecordData.getSubjectiveScore());
        data.setSuccPercent(examRecordData.getSuccPercent());
        data.setAllObjectivePaper(examRecordData.getIsAllObjectivePaper());
        data.setExamStageId(examRecordData.getExamStageId());
        data.setExamStageOrder(examRecordData.getExamStageOrder());
        data.setLastActiveTime(examRecordData.getLastActiveTime());
        data.setContinuedTime(examRecordData.getContinuedTime());
        data.setEnterExamTime(examRecordData.getEnterExamTime());
        data.setSwitchScreenCount(examRecordData.getSwitchScreenCount());

        return data;
    }

    /**
     * 设置并保存考试记录的同步状态
     *
     * @param examRecordDataId
     */
    private void setAndSaveExamRecordDataSyncStatus(Long examRecordDataId) {
        ExamRecordData examRecordData = examRecordDataService.getExamRecordDataCache(examRecordDataId);

        if (SyncStatus.SYNCED == examRecordData.getSyncStatus()) {
            return;
        }

        examRecordData.setSyncStatus(SyncStatus.SYNCED);
        examRecordDataService.saveExamRecordDataCache(examRecordDataId, examRecordData);

        try {
            //同步更新数据库表中的同步状态
            UpdatePartialExamRecordReq uperReq = new UpdatePartialExamRecordReq(examRecordDataId);
            uperReq.setSyncStatus(SyncStatus.SYNCED.name());
            examRecordDataCloudService.updatePartialExamRecord(uperReq);
        } catch (Exception e) {
            this.errorLog("更新数据库表中的同步状态失败", examRecordDataId, e);
        }
    }

    private void debugLog(String msg, Long examRecordDataId) {
        if (LOG.isDebugEnabled()) {
            LOG.debug("[SYNC-EXAM-DATA-EXECUTOR-" + examRecordDataId + "]：" + msg);
        }
    }

    private void errorLog(String msg, Long examRecordDataId, Exception e) {
        if (LOG.isErrorEnabled()) {
            LOG.error("[SYNC-EXAM-DATA-EXECUTOR-" + examRecordDataId + "]：" + msg, e);
        }
    }

}
